Package: pysparklyr
Title: Provides a 'PySpark' Back-End for the 'sparklyr' Package
Version: 0.2.1
Authors@R: c(
    person("Edgar", "Ruiz", , "edgar@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: It enables 'sparklyr' to integrate with 'Spark Connect', and
    'Databricks Connect' by providing a wrapper over the 'PySpark'
    'python' library.
License: MIT + file LICENSE
URL: https://github.com/mlverse/pysparklyr
BugReports: https://github.com/mlverse/pysparklyr/issues
Depends: R (>= 4.1)
Imports: arrow, cli, DBI, dbplyr, dplyr, fs, glue, httr2, lifecycle,
        methods, processx, purrr, reticulate (>= 1.44.0), rlang,
        rstudioapi, sparklyr (>= 1.9.4), tidyr, tidyselect, vctrs,
        uuid, withr, connectcreds
Suggests: vcr, crayon, R6, testthat (>= 3.0.0), tibble, rsconnect,
        rsample, workflows, tune, parsnip, dials, tailor, recipes
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-11-12
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-20 17:31:03 UTC; edgar
Author: Edgar Ruiz [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Edgar Ruiz <edgar@posit.co>
Repository: CRAN
Date/Publication: 2026-04-20 20:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 20:47:13 UTC; unix
