# Generated by roxygen2: do not edit by hand

S3method("[",tbl_pyspark)
S3method(collect,spark_pyobj)
S3method(collect,tbl_pyspark)
S3method(compute,tbl_pyspark)
S3method(connection_is_open,pyspark_connection)
S3method(ft_binarizer,ml_connect_pipeline)
S3method(ft_binarizer,pyspark_connection)
S3method(ft_binarizer,tbl_pyspark)
S3method(ft_bucketed_random_projection_lsh,ml_connect_pipeline)
S3method(ft_bucketed_random_projection_lsh,pyspark_connection)
S3method(ft_bucketed_random_projection_lsh,tbl_pyspark)
S3method(ft_bucketizer,ml_connect_pipeline)
S3method(ft_bucketizer,pyspark_connection)
S3method(ft_bucketizer,tbl_pyspark)
S3method(ft_count_vectorizer,ml_connect_pipeline)
S3method(ft_count_vectorizer,pyspark_connection)
S3method(ft_count_vectorizer,tbl_pyspark)
S3method(ft_dct,ml_connect_pipeline)
S3method(ft_dct,pyspark_connection)
S3method(ft_dct,tbl_pyspark)
S3method(ft_discrete_cosine_transform,ml_connect_pipeline)
S3method(ft_discrete_cosine_transform,pyspark_connection)
S3method(ft_discrete_cosine_transform,tbl_pyspark)
S3method(ft_elementwise_product,ml_connect_pipeline)
S3method(ft_elementwise_product,pyspark_connection)
S3method(ft_elementwise_product,tbl_pyspark)
S3method(ft_feature_hasher,ml_connect_pipeline)
S3method(ft_feature_hasher,pyspark_connection)
S3method(ft_feature_hasher,tbl_pyspark)
S3method(ft_hashing_tf,ml_connect_pipeline)
S3method(ft_hashing_tf,pyspark_connection)
S3method(ft_hashing_tf,tbl_pyspark)
S3method(ft_idf,ml_connect_pipeline)
S3method(ft_idf,pyspark_connection)
S3method(ft_idf,tbl_pyspark)
S3method(ft_imputer,ml_connect_pipeline)
S3method(ft_imputer,pyspark_connection)
S3method(ft_imputer,tbl_pyspark)
S3method(ft_index_to_string,ml_connect_pipeline)
S3method(ft_index_to_string,pyspark_connection)
S3method(ft_index_to_string,tbl_pyspark)
S3method(ft_interaction,ml_connect_pipeline)
S3method(ft_interaction,pyspark_connection)
S3method(ft_interaction,tbl_pyspark)
S3method(ft_max_abs_scaler,ml_connect_pipeline)
S3method(ft_max_abs_scaler,pyspark_connection)
S3method(ft_max_abs_scaler,tbl_pyspark)
S3method(ft_min_max_scaler,ml_connect_pipeline)
S3method(ft_min_max_scaler,pyspark_connection)
S3method(ft_min_max_scaler,tbl_pyspark)
S3method(ft_minhash_lsh,ml_connect_pipeline)
S3method(ft_minhash_lsh,pyspark_connection)
S3method(ft_minhash_lsh,tbl_pyspark)
S3method(ft_ngram,ml_connect_pipeline)
S3method(ft_ngram,pyspark_connection)
S3method(ft_ngram,tbl_pyspark)
S3method(ft_normalizer,ml_connect_pipeline)
S3method(ft_normalizer,pyspark_connection)
S3method(ft_normalizer,tbl_pyspark)
S3method(ft_one_hot_encoder,ml_connect_pipeline)
S3method(ft_one_hot_encoder,pyspark_connection)
S3method(ft_one_hot_encoder,tbl_pyspark)
S3method(ft_pca,ml_connect_pipeline)
S3method(ft_pca,pyspark_connection)
S3method(ft_pca,tbl_pyspark)
S3method(ft_polynomial_expansion,ml_connect_pipeline)
S3method(ft_polynomial_expansion,pyspark_connection)
S3method(ft_polynomial_expansion,tbl_pyspark)
S3method(ft_quantile_discretizer,ml_connect_pipeline)
S3method(ft_quantile_discretizer,pyspark_connection)
S3method(ft_quantile_discretizer,tbl_pyspark)
S3method(ft_r_formula,ml_connect_pipeline)
S3method(ft_r_formula,pyspark_connection)
S3method(ft_r_formula,tbl_pyspark)
S3method(ft_regex_tokenizer,ml_connect_pipeline)
S3method(ft_regex_tokenizer,pyspark_connection)
S3method(ft_regex_tokenizer,tbl_pyspark)
S3method(ft_robust_scaler,ml_connect_pipeline)
S3method(ft_robust_scaler,pyspark_connection)
S3method(ft_robust_scaler,tbl_pyspark)
S3method(ft_sql_transformer,ml_connect_pipeline)
S3method(ft_sql_transformer,pyspark_connection)
S3method(ft_sql_transformer,tbl_pyspark)
S3method(ft_standard_scaler,ml_connect_pipeline)
S3method(ft_standard_scaler,pyspark_connection)
S3method(ft_standard_scaler,tbl_pyspark)
S3method(ft_stop_words_remover,ml_connect_pipeline)
S3method(ft_stop_words_remover,pyspark_connection)
S3method(ft_stop_words_remover,tbl_pyspark)
S3method(ft_string_indexer,ml_connect_pipeline)
S3method(ft_string_indexer,pyspark_connection)
S3method(ft_string_indexer,tbl_pyspark)
S3method(ft_tokenizer,ml_connect_pipeline)
S3method(ft_tokenizer,pyspark_connection)
S3method(ft_tokenizer,tbl_pyspark)
S3method(ft_vector_assembler,ml_connect_pipeline)
S3method(ft_vector_assembler,pyspark_connection)
S3method(ft_vector_assembler,tbl_pyspark)
S3method(ft_vector_indexer,ml_connect_pipeline)
S3method(ft_vector_indexer,pyspark_connection)
S3method(ft_vector_indexer,tbl_pyspark)
S3method(ft_vector_slicer,ml_connect_pipeline)
S3method(ft_vector_slicer,pyspark_connection)
S3method(ft_vector_slicer,tbl_pyspark)
S3method(ft_word2vec,ml_connect_pipeline)
S3method(ft_word2vec,pyspark_connection)
S3method(ft_word2vec,tbl_pyspark)
S3method(head,tbl_pyspark)
S3method(hive_context,pyspark_connection)
S3method(invoke,ml_connect_estimator)
S3method(invoke,ml_connect_model)
S3method(invoke,ml_connect_transformer)
S3method(invoke,pyspark_connection)
S3method(invoke,python.builtin.object)
S3method(invoke,spark_pyobj)
S3method(invoke,tbl_pyspark)
S3method(invoke_static,pyspark_connection)
S3method(ml_aft_survival_regression,ml_connect_pipeline)
S3method(ml_aft_survival_regression,pyspark_connection)
S3method(ml_aft_survival_regression,tbl_pyspark)
S3method(ml_binary_classification_evaluator,pyspark_connection)
S3method(ml_binary_classification_evaluator,tbl_pyspark)
S3method(ml_bisecting_kmeans,ml_connect_pipeline)
S3method(ml_bisecting_kmeans,pyspark_connection)
S3method(ml_bisecting_kmeans,tbl_pyspark)
S3method(ml_clustering_evaluator,pyspark_connection)
S3method(ml_clustering_evaluator,tbl_pyspark)
S3method(ml_cross_validator,pyspark_connection)
S3method(ml_decision_tree_classifier,ml_connect_pipeline)
S3method(ml_decision_tree_classifier,pyspark_connection)
S3method(ml_decision_tree_classifier,tbl_pyspark)
S3method(ml_decision_tree_regressor,ml_connect_pipeline)
S3method(ml_decision_tree_regressor,pyspark_connection)
S3method(ml_decision_tree_regressor,tbl_pyspark)
S3method(ml_fit,ml_connect_cross_validator)
S3method(ml_fit,ml_connect_pipeline)
S3method(ml_gbt_classifier,ml_connect_pipeline)
S3method(ml_gbt_classifier,pyspark_connection)
S3method(ml_gbt_classifier,tbl_pyspark)
S3method(ml_gbt_regressor,ml_connect_pipeline)
S3method(ml_gbt_regressor,pyspark_connection)
S3method(ml_gbt_regressor,tbl_pyspark)
S3method(ml_generalized_linear_regression,ml_connect_pipeline)
S3method(ml_generalized_linear_regression,pyspark_connection)
S3method(ml_generalized_linear_regression,tbl_pyspark)
S3method(ml_isotonic_regression,ml_connect_pipeline)
S3method(ml_isotonic_regression,pyspark_connection)
S3method(ml_isotonic_regression,tbl_pyspark)
S3method(ml_kmeans,ml_connect_pipeline)
S3method(ml_kmeans,pyspark_connection)
S3method(ml_kmeans,tbl_pyspark)
S3method(ml_linear_regression,ml_connect_pipeline)
S3method(ml_linear_regression,pyspark_connection)
S3method(ml_linear_regression,tbl_pyspark)
S3method(ml_logistic_regression,ml_connect_pipeline)
S3method(ml_logistic_regression,pyspark_connection)
S3method(ml_logistic_regression,tbl_pyspark)
S3method(ml_multiclass_classification_evaluator,pyspark_connection)
S3method(ml_multiclass_classification_evaluator,tbl_pyspark)
S3method(ml_pipeline,pyspark_connection)
S3method(ml_predict,ml_connect_model)
S3method(ml_random_forest_classifier,ml_connect_pipeline)
S3method(ml_random_forest_classifier,pyspark_connection)
S3method(ml_random_forest_classifier,tbl_pyspark)
S3method(ml_random_forest_regressor,ml_connect_pipeline)
S3method(ml_random_forest_regressor,pyspark_connection)
S3method(ml_random_forest_regressor,tbl_pyspark)
S3method(ml_regression_evaluator,pyspark_connection)
S3method(ml_regression_evaluator,tbl_pyspark)
S3method(ml_save,ml_connect_pipeline_stage)
S3method(ml_transform,ml_connect_pipeline_model)
S3method(pivot_longer,tbl_pyspark)
S3method(print,ml_connect_cross_validator_model)
S3method(print,ml_connect_estimator)
S3method(print,ml_connect_model)
S3method(print,ml_output_params)
S3method(print,spark_pyobj)
S3method(python_obj_get,default)
S3method(python_obj_get,ml_connect_model)
S3method(python_obj_get,ml_connect_pipeline)
S3method(python_obj_get,ml_connect_pipeline_model)
S3method(python_obj_get,python.builtin.object)
S3method(python_obj_get,spark_pyobj)
S3method(python_obj_get,tbl_pyspark)
S3method(same_src,pyspark_connection)
S3method(sample_frac,tbl_pyspark)
S3method(sample_n,tbl_pyspark)
S3method(sdf_copy_to,pyspark_connection)
S3method(sdf_read_column,spark_pyjobj)
S3method(sdf_register,spark_pyobj)
S3method(sdf_schema,tbl_pyspark)
S3method(spark_apply,tbl_pyspark)
S3method(spark_connect_method,spark_method_databricks_connect)
S3method(spark_connect_method,spark_method_snowpark_connect)
S3method(spark_connect_method,spark_method_spark_connect)
S3method(spark_connection,pyspark_connection)
S3method(spark_connection,spark_pyobj)
S3method(spark_dataframe,pyspark_connection)
S3method(spark_dataframe,spark_pyobj)
S3method(spark_dataframe,tbl_pyspark)
S3method(spark_ide_columns,pyspark_connection)
S3method(spark_ide_objects,pyspark_connection)
S3method(spark_ide_preview,pyspark_connection)
S3method(spark_integ_test_skip,connect_databricks)
S3method(spark_integ_test_skip,connect_spark)
S3method(spark_jobj,ml_connect_model)
S3method(spark_log,pyspark_connection)
S3method(spark_read_csv,pyspark_connection)
S3method(spark_read_json,pyspark_connection)
S3method(spark_read_orc,pyspark_connection)
S3method(spark_read_parquet,pyspark_connection)
S3method(spark_read_text,pyspark_connection)
S3method(spark_session,pyspark_connection)
S3method(spark_version,pyspark_connection)
S3method(spark_version,spark_pyobj)
S3method(spark_web,pyspark_connection)
S3method(spark_write_csv,tbl_pyspark)
S3method(spark_write_delta,tbl_pyspark)
S3method(spark_write_json,tbl_pyspark)
S3method(spark_write_orc,tbl_pyspark)
S3method(spark_write_parquet,tbl_pyspark)
S3method(spark_write_table,tbl_pyspark)
S3method(spark_write_text,tbl_pyspark)
S3method(tbl,pyspark_connection)
S3method(tbl_ptype,tbl_pyspark)
S3method(tidyselect_data_has_predicates,tbl_pyspark)
S3method(tune_grid_spark,pyspark_connection)
export(connection_databricks_shinyapp)
export(deploy_databricks)
export(install_databricks)
export(install_pyspark)
export(installed_components)
export(ml_prepare_dataset)
export(pyspark_config)
export(requirements_write)
export(spark_connect_service_start)
export(spark_connect_service_stop)
import(DBI)
import(cli)
import(dbplyr)
import(fs)
import(glue)
import(httr2)
import(reticulate)
importFrom(connectcreds,connect_viewer_token)
importFrom(connectcreds,has_viewer_token)
importFrom(dplyr,as_tibble)
importFrom(dplyr,collect)
importFrom(dplyr,compute)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_vars)
importFrom(dplyr,mutate)
importFrom(dplyr,same_src)
importFrom(dplyr,sample_frac)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,slice_sample)
importFrom(dplyr,tbl)
importFrom(dplyr,tbl_ptype)
importFrom(dplyr,tibble)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setOldClass)
importFrom(processx,process)
importFrom(purrr,as_mapper)
importFrom(purrr,discard)
importFrom(purrr,flatten)
importFrom(purrr,imap)
importFrom(purrr,imap_chr)
importFrom(purrr,iwalk)
importFrom(purrr,list_rbind)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_chr)
importFrom(purrr,map_lgl)
importFrom(purrr,map_vec)
importFrom(purrr,pmap_chr)
importFrom(purrr,reduce)
importFrom(purrr,transpose)
importFrom(purrr,walk)
importFrom(rlang,`!!!`)
importFrom(rlang,`!!`)
importFrom(rlang,`%||%`)
importFrom(rlang,abort)
importFrom(rlang,arg_match)
importFrom(rlang,as_utf8_character)
importFrom(rlang,enquo)
importFrom(rlang,exec)
importFrom(rlang,is_character)
importFrom(rlang,is_missing)
importFrom(rlang,is_string)
importFrom(rlang,parse_exprs)
importFrom(rlang,quo_is_null)
importFrom(rlang,set_names)
importFrom(rlang,sym)
importFrom(rlang,warn)
importFrom(rstudioapi,getSourceEditorContext)
importFrom(rstudioapi,jobRunScript)
importFrom(rstudioapi,showQuestion)
importFrom(sparklyr,connection_is_open)
importFrom(sparklyr,ft_binarizer)
importFrom(sparklyr,ft_bucketed_random_projection_lsh)
importFrom(sparklyr,ft_bucketizer)
importFrom(sparklyr,ft_count_vectorizer)
importFrom(sparklyr,ft_dct)
importFrom(sparklyr,ft_discrete_cosine_transform)
importFrom(sparklyr,ft_elementwise_product)
importFrom(sparklyr,ft_feature_hasher)
importFrom(sparklyr,ft_hashing_tf)
importFrom(sparklyr,ft_idf)
importFrom(sparklyr,ft_imputer)
importFrom(sparklyr,ft_index_to_string)
importFrom(sparklyr,ft_interaction)
importFrom(sparklyr,ft_max_abs_scaler)
importFrom(sparklyr,ft_min_max_scaler)
importFrom(sparklyr,ft_minhash_lsh)
importFrom(sparklyr,ft_ngram)
importFrom(sparklyr,ft_normalizer)
importFrom(sparklyr,ft_one_hot_encoder)
importFrom(sparklyr,ft_pca)
importFrom(sparklyr,ft_polynomial_expansion)
importFrom(sparklyr,ft_quantile_discretizer)
importFrom(sparklyr,ft_r_formula)
importFrom(sparklyr,ft_regex_tokenizer)
importFrom(sparklyr,ft_robust_scaler)
importFrom(sparklyr,ft_sql_transformer)
importFrom(sparklyr,ft_standard_scaler)
importFrom(sparklyr,ft_stop_words_remover)
importFrom(sparklyr,ft_string_indexer)
importFrom(sparklyr,ft_tokenizer)
importFrom(sparklyr,ft_vector_assembler)
importFrom(sparklyr,ft_vector_indexer)
importFrom(sparklyr,ft_vector_slicer)
importFrom(sparklyr,ft_word2vec)
importFrom(sparklyr,hive_context)
importFrom(sparklyr,invoke)
importFrom(sparklyr,invoke_new)
importFrom(sparklyr,invoke_static)
importFrom(sparklyr,ml_aft_survival_regression)
importFrom(sparklyr,ml_binary_classification_evaluator)
importFrom(sparklyr,ml_bisecting_kmeans)
importFrom(sparklyr,ml_clustering_evaluator)
importFrom(sparklyr,ml_cross_validator)
importFrom(sparklyr,ml_decision_tree_classifier)
importFrom(sparklyr,ml_decision_tree_regressor)
importFrom(sparklyr,ml_fit)
importFrom(sparklyr,ml_gbt_classifier)
importFrom(sparklyr,ml_gbt_regressor)
importFrom(sparklyr,ml_generalized_linear_regression)
importFrom(sparklyr,ml_isotonic_regression)
importFrom(sparklyr,ml_kmeans)
importFrom(sparklyr,ml_linear_regression)
importFrom(sparklyr,ml_load)
importFrom(sparklyr,ml_logistic_regression)
importFrom(sparklyr,ml_multiclass_classification_evaluator)
importFrom(sparklyr,ml_pipeline)
importFrom(sparklyr,ml_predict)
importFrom(sparklyr,ml_random_forest_classifier)
importFrom(sparklyr,ml_random_forest_regressor)
importFrom(sparklyr,ml_regression_evaluator)
importFrom(sparklyr,ml_save)
importFrom(sparklyr,ml_transform)
importFrom(sparklyr,random_string)
importFrom(sparklyr,sdf_copy_to)
importFrom(sparklyr,sdf_read_column)
importFrom(sparklyr,sdf_register)
importFrom(sparklyr,sdf_schema)
importFrom(sparklyr,spark_apply)
importFrom(sparklyr,spark_connect_method)
importFrom(sparklyr,spark_connection)
importFrom(sparklyr,spark_dataframe)
importFrom(sparklyr,spark_ide_columns)
importFrom(sparklyr,spark_ide_connection_updated)
importFrom(sparklyr,spark_ide_objects)
importFrom(sparklyr,spark_ide_preview)
importFrom(sparklyr,spark_install_find)
importFrom(sparklyr,spark_integ_test_skip)
importFrom(sparklyr,spark_jobj)
importFrom(sparklyr,spark_log)
importFrom(sparklyr,spark_read_csv)
importFrom(sparklyr,spark_read_json)
importFrom(sparklyr,spark_read_orc)
importFrom(sparklyr,spark_read_parquet)
importFrom(sparklyr,spark_read_text)
importFrom(sparklyr,spark_session)
importFrom(sparklyr,spark_table_name)
importFrom(sparklyr,spark_version)
importFrom(sparklyr,spark_web)
importFrom(sparklyr,spark_write_csv)
importFrom(sparklyr,spark_write_delta)
importFrom(sparklyr,spark_write_json)
importFrom(sparklyr,spark_write_orc)
importFrom(sparklyr,spark_write_parquet)
importFrom(sparklyr,spark_write_table)
importFrom(sparklyr,spark_write_text)
importFrom(sparklyr,tune_grid_spark)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(tidyr,pivot_longer)
importFrom(tidyselect,matches)
importFrom(tidyselect,tidyselect_data_has_predicates)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,installed.packages)
importFrom(utils,menu)
importFrom(utils,type.convert)
importFrom(uuid,UUIDgenerate)
importFrom(vctrs,vec_as_names)
