# Snowpark Connect (Snowflake)

    Code
      sc_out
    Output
      $conn
      $conn[[1]]
      $conn[[1]]$user
      [1] "test@user.com"
      
      $conn[[1]]$password
      [1] "testtoken"
      
      $conn[[1]]$warehouse
      [1] "testwh"
      
      $conn[[1]]$database
      [1] "testdb"
      
      $conn[[1]]$schema
      [1] "testschema"
      
      $conn[[1]]$account
      [1] "testaccount"
      
      
      
      $master_label
      Snowpark Connect - testaccount
      
      $con_class
      [1] "connect_snowflake"
      
      $cluster_id
      NULL
      
      $method
      [1] "snowpark_connect"
      
      $config
      NULL
      
      $misc
      $misc$sql_catalogs
      [1] "show databases"
      
      $misc$sql_tables_schema
      [1] "show tables in {schema}"
      
      $misc$sql_tables_catalog_schema
      [1] "show tables in {catalog}.{schema}"
      
      $misc$sql_schemas_catalog
      [1] "show schemas in database {catalog}"
      
      $misc$sql_schemas
      [1] "show schemas"
      
      
      $quote
      [1] ""
      

