# Schema outputs as expected

    Code
      sdf_schema(tbl_mtcars)
    Output
      $mpg
      $mpg$name
      [1] "mpg"
      
      $mpg$type
      [1] "DoubleType"
      
      
      $cyl
      $cyl$name
      [1] "cyl"
      
      $cyl$type
      [1] "DoubleType"
      
      
      $disp
      $disp$name
      [1] "disp"
      
      $disp$type
      [1] "DoubleType"
      
      
      $hp
      $hp$name
      [1] "hp"
      
      $hp$type
      [1] "DoubleType"
      
      
      $drat
      $drat$name
      [1] "drat"
      
      $drat$type
      [1] "DoubleType"
      
      
      $wt
      $wt$name
      [1] "wt"
      
      $wt$type
      [1] "DoubleType"
      
      
      $qsec
      $qsec$name
      [1] "qsec"
      
      $qsec$type
      [1] "DoubleType"
      
      
      $vs
      $vs$name
      [1] "vs"
      
      $vs$type
      [1] "DoubleType"
      
      
      $am
      $am$name
      [1] "am"
      
      $am$type
      [1] "DoubleType"
      
      
      $gear
      $gear$name
      [1] "gear"
      
      $gear$type
      [1] "DoubleType"
      
      
      $carb
      $carb$name
      [1] "carb"
      
      $carb$type
      [1] "DoubleType"
      
      

