# Cross validator works

    Code
      class(cv)
    Output
      [1] "ml_connect_cross_validator" "ml_cross_validator"        
      [3] "ml_connect_estimator"       "ml_estimator"              
      [5] "ml_pipeline_stage"         

---

    Code
      class(tuning_model)
    Output
      [1] "ml_connect_cross_validator_model" "ml_cross_validator_model"        

---

    Code
      use_test_pull(metrics)
    Output
      [1] 0.0 0.5 1.0

